<template>
    <layout-form>
        <el-form :model="form" :loading="loading" :rules="rules" size='small' style="width:100%" label-width='120px' ref="form">
            <el-row>
            <#list columns as column>
                <#if column.isPrimaryKey != 1 && !column.isSuperColumn(column.javaField)>
                    <#assign javaField=column.javaField />
                    <#assign parentheseIndex=column.columnComment?index_of("（") />
                    <#if parentheseIndex != -1>
                        <#assign comment=column.columnComment[0..parentheseIndex] />
                    <#else >
                        <#assign comment=column.columnComment />
                    </#if>
                    <#if column.htmlType == 1 && !column.columnType?contains("tinyint")>
                        <Field label="${comment}" prop="${column.javaField}" v-model="form.${column.javaField}" placeholder="请输入${column.columnComment}"/>
                    <#elseif column.htmlType == 2>
                        <Field label="${comment}" prop="${column.javaField}" v-model="form.${column.javaField}" type="textarea" placeholder="请输入${column.columnComment}"/>
                    <#elseif column.htmlType == 3 ||column.columnType?contains("tinyint")>
                        <Field label="${comment}" prop="${column.javaField}" v-model="form.${column.javaField}" type="select" :enumData="dict.${column.javaField}" placeholder="请选择${column.columnComment}"/>
                    <#elseif column.htmlType == 4>
                        <Field label="${comment}" prop="${column.javaField}" v-model="form.${column.javaField}" type="checkbox" :enumData="dict.${column.javaField}" />
                    <#elseif column.htmlType == 5>
                        <Field label="${comment}" prop="${column.javaField}" v-model="form.${column.javaField}" type="radio" :enumData="dict.${column.javaField}" />
                    <#elseif column.htmlType == 6>
                        <Field label="${comment}" prop="${column.javaField}" v-model="form.${column.javaField}" type="date" />
                    <#elseif column.htmlType == 7>
                        <el-col span="12">
                            <el-form-item label="${comment}">
                                <imageUpload v-model="form.${column.javaField}" prePath="/uploadimage"/>
                            </el-form-item>
                        </el-col>
                    <#elseif column.htmlType == 8>
                        <el-col span="12">
                            <el-form-item label="${comment}">
                                <fileUpload v-model="form.${column.javaField}"  prePath="/uploadfile" />
                            </el-form-item>
                        </el-col>
                    <#elseif column.htmlType == 9>
                        <el-col span="12">
                            <el-form-item label="${comment}">
                                <editor v-model="form.${column.javaField}" :min-height="192"/>
                            </el-form-item>
                        </el-col>
                    <#else >
                        <Field label="${column.columnComment}" prop="${column.javaField}" v-model="form.${column.javaField}" placeholder="请输入${column.columnComment}"/>
                    </#if>
                </#if>
            </#list>
            </el-row>
        <#if table.tplCategory=="sub">
            <el-divider content-position="center">${subTable.functionName}信息</el-divider>
            <br/>
            <el-row :gutter="10" class="mb8">
                <el-col :span="1.5">
                    <el-button type="primary" icon="el-icon-plus" size="mini" @click="handleAdd${subClassName}">添加</el-button>
                </el-col>
                <el-col :span="1.5">
                    <el-button type="danger" icon="el-icon-delete" size="mini" @click="handleDelete${subClassName}">删除</el-button>
                </el-col>
            </el-row>

            <el-table :data="${subclassName}List" :row-class-name="row${subClassName}Index" @selection-change="handle${subClassName}SelectionChange" ref="${subclassName}">
                <el-table-column type="selection" width="50" align="center" />
                <el-table-column label="序号" align="center" prop="index" width="50"/>
                <#list subTable.columns as column >
                    <#assign javaField=column.javaField />
                    <#assign parentheseIndex=column.columnComment?index_of("（") />
                    <#if parentheseIndex != -1>
                        <#assign comment=column.columnComment[0..parentheseIndex] />
                        <#else >
                        <#assign comment=column.columnComment />
                    </#if>
                    <#if column.isPrimaryKey==1 || javaField == subTableFkclassName>
                    <#elseif "" != javaField>
                <el-table-column label="${comment}" prop="${javaField}">
                    <template slot-scope="scope">
                        <el-input v-model="scope.row.${javaField}" placeholder="请输入${comment}" />
                    </template>
                </el-table-column>
                    </#if>
                </#list>
            </el-table>
        </#if>

        <#if table.tplCategory=="subone">
            <el-divider content-position="center">${subTable.functionName}信息</el-divider>
            <br/>
            <el-row>
            <#list subTable.columns as column >
            <#assign javaField=column.javaField />
            <#assign parentheseIndex=column.columnComment?index_of("（") />
            <#if parentheseIndex != -1>
                <#assign comment=column.columnComment[0..parentheseIndex] />
            <#else >
                <#assign comment=column.columnComment />
            </#if>
            <#if column.isPrimaryKey==1 || javaField == subTableFkclassName>
            <#elseif "" != javaField>
                <el-col :span="12">
                    <el-form-item label="${comment}" placeholder="请输入${comment}">
                        <el-input v-model="form.${subclassName}Entity.${javaField}"></el-input>
                    </el-form-item>
                </el-col>

                <#if column.htmlType == 1 && !column.columnType?contains("tinyint")>
                    <el-form-item label="${comment}" placeholder="请输入${comment}">
                        <el-input v-model="form.${subclassName}Entity.${javaField}"></el-input>
                    </el-form-item>
                <#elseif column.htmlType == 2>
                    <el-form-item label="${comment}" placeholder="请输入${comment}">
                        <el-input v-model="form.${subclassName}Entity.${javaField}" type="textarea" :rows="2"></el-input>
                    </el-form-item>
                <#elseif column.htmlType == 3 ||column.columnType?contains("tinyint")>
                    <el-form-item label="${comment}" prop="${column.javaField}" placeholder="请选择${comment}>
                        <el-select v-model="form.${subclassName}Entity.${javaField}" placeholder="请选择${comment}" clearable size="small">
                            <el-option
                                    v-for="(label, value) in dict.${column.javaField}"
                                    :key="value"
                                    :label="label"
                                    :value="value"
                            ></el-option>
                        </el-select>
                    </el-form-item>
                <#elseif column.htmlType == 4>
                    <el-form-item label="${comment}" >
                        <el-checkbox-group v-model="form.${subclassName}Entity.${javaField}">
                            <el-checkbox
                                    v-for='(label, value) in dict.${column.javaField}'
                                    :key='value'
                                    :label="value"
                            >{{label}}</el-checkbox>
                        </el-checkbox-group>
                    </el-form-item>
                <#elseif column.htmlType == 5>
                    <el-form-item label="${comment}" >
                        <el-radio-group  v-model="form.${subclassName}Entity.${javaField}">
                            <el-radio
                                    v-for='(label, value) in dict.${column.javaField}'
                                    :key='value'
                                    :label="value"
                            >{{label}}</el-radio>
                        </el-radio-group>
                    </el-form-item>
                <#elseif column.htmlType == 6>
                    <el-form-item label="${comment}" >
                        <el-date-picker  type="date" value-format="yyyy-MM-dd HH:mm:ss" v-model="form.${subclassName}Entity.${javaField}"  placeholder="选择日期" ></el-date-picker>
                    </el-form-item>
                <#elseif column.htmlType == 7>
                    <el-form-item label="${comment}">
                        <imageUpload v-model="form.${subclassName}Entity.${javaField}" prePath="/uploadimage"/>
                    </el-form-item>
                <#elseif column.htmlType == 8>
                    <el-form-item label="${comment}">
                        <fileUpload v-model="form.${subclassName}Entity.${javaField}"  prePath="/uploadfile" />
                    </el-form-item>
                <#elseif column.htmlType == 9>
                        <el-form-item label="${comment}">
                            <editor v-model="form.${subclassName}Entity.${javaField}" :min-height="192"/>
                        </el-form-item>
                <#else >
                    <Field label="${column.columnComment}" prop="${column.javaField}" v-model="form.${column.javaField}" placeholder="请输入${column.columnComment}"/>
                </#if>
            </#if>
            </#list>
            </el-row>
        </#if>
            <form-buttons @submit='submitForm'/>
        </el-form>
    </layout-form>
</template>

<script>
    import form from "@/assets/mixins/form";
    <#list columns as column>
    <#if column.isInsert==1 && !column.isSuperColumn(column.javaField) && column.isPrimaryKey!=1 && column.htmlType == 7>
    import ImageUpload from '@/components/ImageUpload';
    <#break>
    </#if>
    </#list>
    <#list columns as column>
    <#if column.isInsert==1 && !column.isSuperColumn(column.javaField) && column.isPrimaryKey!=1 && column.htmlType == 8>
    import FileUpload from '@/components/FileUpload';
        <#break>
        </#if>
        </#list>
    <#list columns as column>
        <#if column.isInsert==1 && !column.isSuperColumn(column.javaField) && column.isPrimaryKey!=1 && column.htmlType == 9>
    import Editor from '@/components/Editor';
        <#break>
    </#if>
    </#list>
    export default {
        mixins: [form],
        components: {
        <#list columns as column>
            <#if column.isInsert==1 && !column.isSuperColumn(column.javaField) && column.isPrimaryKey!=1 && column.htmlType == 7>
            ImageUpload,
            <#break>
            </#if>
        </#list>
        <#list columns as column>
            <#if column.isInsert==1 && !column.isSuperColumn(column.javaField) && column.isPrimaryKey!=1 && column.htmlType == 8>
            FileUpload,
            <#break>
            </#if>
        </#list>
        <#list columns as column>
            <#if column.isInsert==1 && !column.isSuperColumn(column.javaField) && column.isPrimaryKey!=1 && column.htmlType == 9>
            Editor,
            <#break>
            </#if>
        </#list>
        },
        toString:[
            <#list columns as column>
            <#if column.columnType?contains("tinyint")>
            "${column.javaField}",
            </#if>
            </#list>
        ],
        methods: {
    <#if table.tplCategory=="sub">
        /** ${subTable.functionName}序号 */
        row${subClassName}Index({ row, rowIndex }) {
            row.index = rowIndex + 1;
        },
        /** ${subTable.functionName}添加按钮操作 */
        handleAdd${subClassName}() {
            let obj = {};
            <#list subTable.columns as column>
                <#if column.isPrimaryKey==1 || column.javaField == subTableFkclassName>
                <#elseif "" != javaField>
            obj.${column.javaField} = "";
                </#if>
            </#list>
            this.${subclassName}List.push(obj);
        },
        /** ${subTable.functionName}删除按钮操作 */
        handleDelete${subClassName}() {
            if (this.checked${subClassName}.length == 0) {
                this.$alert("请先选择要删除的${subTable.functionName}数据", "提示", { confirmButtonText: "确定", });
            } else {
                this.${subclassName}List.splice(this.checked${subClassName}[0].index - 1, 1);
            }
        },
        /** 单选框选中数据 */
        handle${subClassName}SelectionChange(selection) {
            if (selection.length > 1) {
                this.$refs.${subclassName}.clearSelection();
                this.$refs.${subclassName}.toggleRowSelection(selection.pop());
            } else {
                this.checked${subClassName} = selection;
            }
        },
        // 渲染前置处理
        beforeRender(data) {
            if(data.entity.${subclassName}List) {
                this.${subclassName}List = data.entity.${subclassName}List;
            }
            return data
        },
        <#elseif table.tplCategory=="subone" >
    </#if>

        },
        data() {
            return {
                <#if table.tplCategory=="sub">
                // 子表选中数据
                checked${subClassName}: [],
                // ${subTable.functionName}表格数据
                ${subclassName}List: [],
                </#if>
                rules: {
                    <#list columns as column>
                    <#if column.javaType == "String" && column.isEdit==1>
                    ${column.javaField}: [
                        {<#if column.isRequired==1>required: true,</#if> trigger: "blur" },
                    ],
                    </#if>
                    <#if column.javaType == "Integer" && column.isEdit==1>
                    ${column.javaField}: [
                        {<#if column.isRequired==1>required: true,</#if> type: 'integer',trigger: "blur" },
                    ],
                    </#if>
                    <#if column.javaType == "BigDecimal" && column.isEdit==1>
                    ${column.javaField}: [
                        {<#if column.isRequired==1>required: true,</#if> type: 'number',trigger: "blur" },
                    ],
                    </#if>
                    <#if column.javaType == "Date" &&column.htmlType ==6 && column.isEdit==1>
                    ${column.javaField}: [
                        {<#if column.isRequired==1>required: true,</#if> },
                    ],
                    </#if>
                    </#list>
                }
            };
        }
    };
</script>